@@ -8,6 +8,7 @@ from datacode.models.dtypes.float_type import FloatType
 from datacode.models.dtypes.int_type import IntType
 from datacode.models.dtypes.manager import DataTypeManager
 from datacode.models.dtypes.obj_type import ObjectType
+from datacode.models.dtypes.period_type import PeriodType
 from datacode.models.dtypes.str_type import StringType
 from datacode.models.dtypes.timedelta_type import TimedeltaType
 
@@ -24,13 +25,14 @@ def convert_str_to_data_type_if_necessary(dtype: Union[str, DataType]) -> Option
 
 def convert_str_to_data_type(dtype: str) -> DataType:
     manager = DataTypeManager([
-        BooleanType(),
-        DateType(),
-        DatetimeType(),
-        FloatType(),
-        IntType(),
-        ObjectType(),
-        StringType(),
-        TimedeltaType(),
+        BooleanType,
+        DateType,
+        DatetimeType,
+        FloatType,
+        IntType,
+        ObjectType,
+        StringType,
+        TimedeltaType,
+        PeriodType,
     ])
     return manager.get_by_name(dtype)
