@@ -1,11 +1,13 @@
 import datetime
 import time
+from copy import deepcopy
+from unittest.mock import patch
 
 from pandas.testing import assert_frame_equal
 
 from datacode import DataSource, Column
 import datacode.hooks as dc_hooks
-from tests.pipeline.base import PipelineTest
+from tests.pipeline.base import PipelineTest, source_transform_func2, source_transform_func
 import tests.test_hooks as th
 
 
@@ -118,14 +120,15 @@ class TestDataTransformationPipeline(PipelineTest):
         dmp = self.create_merge_pipeline()
 
         dtp = self.create_transformation_pipeline(source=dmp)
-        self.create_csv_for_2()
-        ds = self.create_source(df=None, location=self.csv_path2, pipeline=dtp)
+        time.sleep(0.01)
+        self.create_csv_for_3()
+        ds = self.create_source(df=None, location=self.csv_path3, pipeline=dtp)
 
         # Should not run pipeline as source is newer
         df = ds.df
 
         dc_hooks.reset_hooks()
-        assert_frame_equal(df, self.test_df2)
+        assert_frame_equal(df, self.test_df3)
         assert th.COUNTER == counter_value  # transform operation not called
         self.assert_ordered_pipeline_operations(dtp, [dmp, dtp])
 
@@ -139,9 +142,14 @@ class TestDataTransformationPipeline(PipelineTest):
         self.create_csv_for_2()
         ds = self.create_source(df=None, location=self.csv_path2, pipeline=dtp)
 
+        before_touch = datetime.datetime.now()
         dtp.touch()
-        assert dtp.last_modified > dmp.last_modified
-        assert dtp.last_modified > ds.last_modified
+        after_touch = datetime.datetime.now()
+        assert dtp.last_modified < after_touch
+        assert dtp.last_modified > before_touch
+        assert dmp.last_modified is None
+        assert dtp.pipeline_last_modified == dmp.pipeline_last_modified
+        assert ds.pipeline_last_modified > ds.last_modified
         # Should run pipeline as was just touched
         df = ds.df
 
@@ -150,6 +158,30 @@ class TestDataTransformationPipeline(PipelineTest):
         assert th.COUNTER == counter_value + 1  # transform operation called once
         self.assert_ordered_pipeline_operations(dtp, [dmp, dtp])
 
+    def test_nested_last_modified_of_pipeline_manually_set_to_be_greater_than_source(self):
+        counter_value = th.COUNTER
+        dc_hooks.on_begin_apply_source_transform = th.increase_counter_hook_return_only_second_arg
+
+        dmp = self.create_merge_pipeline()
+
+        now = datetime.datetime.now()
+        later = now + datetime.timedelta(minutes=5)
+        dtp = self.create_transformation_pipeline(source=dmp, last_modified=later)
+        self.create_csv_for_2()
+        ds = self.create_source(df=None, location=self.csv_path2, pipeline=dtp)
+
+        assert dtp.last_modified == later
+        assert dmp.last_modified is None
+        assert dtp.pipeline_last_modified == dmp.pipeline_last_modified
+        assert ds.pipeline_last_modified > ds.last_modified
+        # Should run pipeline as was manually set last modified in the future
+        df = ds.df
+
+        dc_hooks.reset_hooks()
+        assert_frame_equal(df, self.expect_merged_1_2_both_transformed)
+        assert th.COUNTER == counter_value + 1  # transform operation called once
+        self.assert_ordered_pipeline_operations(dtp, [dmp, dtp])
+
     def test_nested_last_modified_of_source_less_than_pipeline(self):
         counter_value = th.COUNTER
         dc_hooks.on_begin_apply_source_transform = th.increase_counter_hook_return_only_second_arg
@@ -186,9 +218,15 @@ class TestDataTransformationPipeline(PipelineTest):
         self.create_csv()  # now earlier source is more recently modified
         assert dmp.data_sources[0].last_modified > ds.last_modified
 
+        before_touch = datetime.datetime.now()
         ds.touch()
-        assert ds.last_modified > dtp.last_modified
-        assert ds.last_modified > dmp.last_modified
+        after_touch = datetime.datetime.now()
+        assert ds.last_modified < after_touch
+        assert ds.last_modified > before_touch
+        assert dmp.last_modified is None
+        assert dtp.last_modified is None
+        assert ds.pipeline_last_modified < ds.last_modified
+
         # Should not run pipeline as source was touched to be newer
         df = ds.df
 
@@ -197,6 +235,35 @@ class TestDataTransformationPipeline(PipelineTest):
         assert th.COUNTER == counter_value  # transform operation not called
         self.assert_ordered_pipeline_operations(dtp, [dmp, dtp])
 
+    def test_nested_last_modified_of_source_manually_overridden_to_be_greater_than_pipeline(self):
+        counter_value = th.COUNTER
+        dc_hooks.on_begin_apply_source_transform = th.increase_counter_hook_return_only_second_arg
+
+        dmp = self.create_merge_pipeline()
+
+        dtp = self.create_transformation_pipeline(source=dmp)
+        self.create_csv_for_2()
+        now = datetime.datetime.now()
+        later = now + datetime.timedelta(minutes=5)
+        ds = self.create_source(df=None, location=self.csv_path2, pipeline=dtp, last_modified=later)
+
+        time.sleep(0.01)
+        self.create_csv()
+        # now earlier source has csv more recently modified, but this
+        # should be ignored due to manually passing last_modified in the future
+
+        assert ds.last_modified == later
+        assert dmp.last_modified is None
+        assert dtp.last_modified is None
+        assert ds.pipeline_last_modified < ds.last_modified
+        # Should not run pipeline as source was manually set to be newer
+        df = ds.df
+
+        dc_hooks.reset_hooks()
+        assert_frame_equal(df, self.test_df2)
+        assert th.COUNTER == counter_value  # transform operation not called
+        self.assert_ordered_pipeline_operations(dtp, [dmp, dtp])
+
     def test_nested_last_modified_of_source_less_than_earlier_source(self):
         counter_value = th.COUNTER
         dc_hooks.on_begin_apply_source_transform = th.increase_counter_hook_return_only_second_arg
@@ -225,3 +292,43 @@ class TestDataTransformationPipeline(PipelineTest):
         assert th.COUNTER == counter_value + 2  # transform operation called once
         self.assert_all_pipeline_operations_have_pipeline(dtp)
         self.assert_all_pipeline_operations_have_pipeline(dtp2)
+
+    @patch('datacode.models.source.DataSource.last_modified', datetime.datetime(2020, 7, 29))
+    def test_transformation_pipeline_cache(self):
+        counter_value = th.COUNTER
+        dc_hooks.on_begin_apply_source_transform = th.increase_counter_hook_return_only_second_arg
+
+        # Run initial pipeline
+        cols = self.create_columns()
+        dtp = self.create_transformation_pipeline(result_kwargs=dict(columns=cols))
+        dtp.execute()
+
+        assert_frame_equal(dtp.df, self.expect_func_df)
+        self.assert_all_pipeline_operations_have_pipeline(dtp)
+        assert th.COUNTER == counter_value + 1  # transform operation called once
+
+        # Now result should be cached to file. Running a new pipeline with
+        # the same options should load from cache
+        # Options should be checked for equality, so passing deepcopy it should
+        # still cache
+        dtp = self.create_transformation_pipeline(
+            func=deepcopy(source_transform_func), result_kwargs=dict(columns=cols)
+        )
+        dtp.execute()
+
+        assert_frame_equal(dtp.df, self.expect_func_df)
+        self.assert_all_pipeline_operations_have_pipeline(dtp)
+        assert th.COUNTER == counter_value + 1  # transform operation not called again
+
+        # Running with different options should run operations again
+        dtp = self.create_transformation_pipeline(
+            pipeline_kwargs=dict(difficulty=20), result_kwargs=dict(columns=cols)
+        )
+        dtp.execute()
+
+        assert_frame_equal(dtp.df, self.expect_func_df)
+        self.assert_all_pipeline_operations_have_pipeline(dtp)
+        assert th.COUNTER == counter_value + 2  # transform operation called again
+
+        dc_hooks.reset_hooks()
+
